@@ -45,7 +45,6 @@ class MmCorpus(matutils.MmReader, IndexedCorpus):
         call it directly, call `serialize` instead.
         """
         logger.info("storing corpus in Matrix Market format to %s" % fname)
-        return matutils.MmWriter.write_corpus(fname, corpus, index=True, progress_cnt=progress_cnt)
+        num_terms = len(id2word) if id2word is not None else None
+        return matutils.MmWriter.write_corpus(fname, corpus, num_terms=num_terms, index=True, progress_cnt=progress_cnt)
 #endclass MmCorpus
-
-
