@@ -20,6 +20,7 @@ from __future__ import with_statement
 import os, sys, logging
 import threading
 import tempfile
+import Queue
 
 from gensim.models import lsimodel
 from gensim import utils
@@ -41,21 +42,31 @@ class Worker(object):
         self.jobsdone = 0 # how many jobs has this worker completed?
         self.myid = myid # id of this worker in the dispatcher; just a convenience var for easy access/logging TODO remove?
         self.dispatcher = dispatcher
+        self.finished = False
         logger.info("initializing worker #%s" % myid)
         self.model = lsimodel.LsiModel(**model_params)
 
 
     def requestjob(self):
         """
-        Request jobs from the dispatcher in an infinite loop. The requests are
-        blocking, so if there are no jobs available, the thread will wait.
+        Request jobs from the dispatcher, in a perpetual loop until `getstate()` is called.
         """
         if self.model is None:
             raise RuntimeError("worker must be initialized before receiving jobs")
-        job = self.dispatcher.getjob(self.myid) # blocks until a new job is available from the dispatcher
-        logger.info("worker #%s received job #%i" % (self.myid, self.jobsdone))
-        self.processjob(job)
-        self.dispatcher.jobdone(self.myid)
+
+        job = None
+        while job is None and not self.finished:
+            try:
+                job = self.dispatcher.getjob(self.myid)
+            except Queue.Empty:
+                # no new job: try again, unless we're finished with all work
+                continue
+        if job is not None:
+            logger.info("worker #%s received job #%i" % (self.myid, self.jobsdone))
+            self.processjob(job)
+            self.dispatcher.jobdone(self.myid)
+        else:
+            logger.info("worker #%i stopping asking for jobs" % self.myid)
 
 
     @utils.synchronous('lock_update')
@@ -72,9 +83,15 @@ class Worker(object):
         logger.info("worker #%i returning its state after %s jobs" %
                     (self.myid, self.jobsdone))
         assert isinstance(self.model.projection, lsimodel.Projection)
-        result = self.model.projection
+        self.finished = True
+        return self.model.projection
+
+
+    @utils.synchronous('lock_update')
+    def reset(self):
+        logger.info("resetting worker #%i" % self.myid)
         self.model.projection = self.model.projection.empty_like()
-        return result
+        self.finished = False
 
 
     def exit(self):
