@@ -102,6 +102,18 @@ class TestDictionary(unittest.TestCase):
         expected = {0: 3, 1: 3, 2: 3, 3: 3}
         self.assertEqual(d.dfs, expected)
 
+    def test_doc2bow(self):
+        d = Dictionary([["žluťoučký"], ["žluťoučký"]])
+
+        # pass a utf8 string
+        self.assertEqual(d.doc2bow(["žluťoučký"]), [(0, 1)])
+
+        # doc2bow must raise a TypeError if passed a string instead of array of strings by accident
+        self.assertRaises(TypeError, d.doc2bow, "žluťoučký")
+
+        # unicode must be converted to utf8
+        self.assertEqual(d.doc2bow([u'\u017elu\u0165ou\u010dk\xfd']), [(0, 1)])
+
     def test_saveAsText_and_loadFromText(self):
         """`Dictionary` can be saved as textfile and loaded again from textfile. """
         tmpf = get_tmpfile('dict_test.txt')
